 // ***********************************************************
// Use this code at your own risk.
//
// Direct questions and/or fixes to:
//   Illah Nourbakhsh     illah@cs.cmu.edu
//   Carnegie Mellon University Robotics Institute
//
// Last updated:  November 6, 1998
//
// ***********************************************************

#include <stdlib.h>

#include <wtypes.h>
#include "useful.h"

#include <jni.h>
#include "serialPort.h"

HANDLE hCom1;  // serial port handle

// open serial port at specified baud with other appropriate parameters.
// returns:
// 0 on success
// -1 on failure to open serial port
// -2 on failure to read port state
// -3 on failure to set port state
// baud rates are: 1 = 9600; 2=19200; 3=38400; 4=57600; 5=115200; 6=230400
// coms are: 1 = "COM1"; 2 = "COM2" et cetera
int openSerial(int comNum, int baudSpec)
{
	DCB dcb;	
	HANDLE hCom;
	DWORD dwError;
	BOOL fSuccess;
	char comName[10];
	long baudRate;

	switch (comNum) {
		case 1:	strcpy(comName, "COM1");
				break;
		case 2: strcpy(comName, "COM2");
				break;
		case 3: strcpy(comName, "COM3");
				break;
		case 4: strcpy(comName, "COM4");
				break;
		case 5: strcpy(comName, "COM5");
				break;
		case 6: strcpy(comName, "COM6");
				break;
		case 7: strcpy(comName, "COM7");
				break;
		case 8: strcpy(comName, "COM8");
				break;
		case 9: strcpy(comName, "COM9");
				break;
		default: strcpy(comName, "COM1");
	}



	hCom = CreateFile(comName,
		GENERIC_READ | GENERIC_WRITE,
		0,    /* comm devices must be opened w/exclusive-access */
		NULL, /* no security attrs */
		OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
		0,    /* not overlapped I/O */
		NULL  /* hTemplate must be NULL for comm devices */
		);

	if (hCom == INVALID_HANDLE_VALUE) {
		dwError = GetLastError();

		return (-1);
	}

	/*
	 * Omit the call to SetupComm to use the default queue sizes.
	 * Get the current configuration.
	*/

	fSuccess = GetCommState(hCom, &dcb);

	if (!fSuccess) {
		return (-2);
	}

	// baud rates are: 1 = 9600; 2=19200; 3=38400; 4=57600; 5=115200; 6=230400
	switch (baudSpec) {
		case 1:	baudRate = 9600;
				break;
		case 2: baudRate = 19200;
				break;
		case 3: baudRate = 38400;
				break;
		case 4: baudRate = 57600;
				break;
		case 5: baudRate = 115200;
				break;
		case 6: baudRate = 230400;
				break;
		default: baudRate = 9600;
	}

	/* Fill in the DCB: baud=whatever, 8 data bits, no parity, 1 stop bit. */

	dcb.BaudRate = baudRate; 
	//dcb.BaudRate = 38400; 

	dcb.fBinary = TRUE;
	dcb.fParity = FALSE;
	
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;

	fSuccess = SetCommState(hCom, &dcb);

	if (!fSuccess) {
		return (-3);
	}

	hCom1 = hCom;
	return (0);
}

int closeSerial() {
	CloseHandle(hCom1);
	return (0);
}



// sends a byte to the serial port.  returns 0 on success and -1 on failure.
int sendByte(char c)
{
	int numBytes = 2;

//	PurgeComm(hCom1, PURGE_RXCLEAR);
//	FlushFileBuffers(hCom1);

//	printf("Entering sendByte with parameter = %c\n", c);

	if (!WriteFile(hCom1, (void *) &c, 
				   1, &numBytes, NULL) || 
		numBytes != 1) {
		return -1;
	}

//	FlushFileBuffers(hCom1);

	return 0;
}

/* sets the timeout for reading a byte 
	returns:
	-6 on failure to set timeout
	0 on success
	(timeout is specified in milliseconds)
	*/
int setReadTimeout(int initialTimeout)
{
	
	COMMTIMEOUTS timeouts;

	timeouts.ReadIntervalTimeout = 0;
	timeouts.ReadTotalTimeoutMultiplier = initialTimeout;
	timeouts.ReadTotalTimeoutConstant = 0;

	if (!SetCommTimeouts(hCom1, &timeouts))
		return (-6);

	return 0;
}


// read byte from serial port.
// returns 0 on success, -1 on failure or timeout.
int readByte(char *ptr)
{
	int numread=0;

	ReadFile(hCom1, ptr, 1, &numread, NULL);

	if (numread != 1) 
		return -1;
	
	return 0;
}


/* ************* Now the Java interface code ******************** */

JNIEXPORT jint JNICALL Java_serialPort_openSerial
(JNIEnv * jnienv, jobject javaobj, jint comNum, jint baudSpec) {
	int retval;
	retval = openSerial(comNum, baudSpec);
	return retval;
}

JNIEXPORT jint JNICALL Java_serialPort_closeSerial
(JNIEnv * jnienv, jobject javaobj) {
	int retval;
	retval = closeSerial();
	return retval;
}

JNIEXPORT jint JNICALL Java_serialPort_sendByte
(JNIEnv * jnienv, jobject javaobj, jint toSend) {
	int retval;
	if ((toSend < 0) || (toSend > 255)) return -1;
	retval = sendByte((char)toSend);
	return retval;
}

JNIEXPORT jint JNICALL Java_serialPort_setReadTimeout
(JNIEnv * jnienv, jobject javaobj, jint timeoutSpec) {
	int retval;
	retval = setReadTimeout(timeoutSpec);
	return retval;
}

JNIEXPORT jint JNICALL Java_serialPort_readByte
(JNIEnv * jnienv, jobject javaobj) {
	int retval;
	unsigned char theByte;
	retval = readByte(&theByte);
	if (retval == -1) return -1;
	else return (unsigned int)theByte;
}
