#include <jni.h>
#include "RobotController.h"
#include "interface.h"


// copy state from the State global here into
// Java's state array
// returncode is what goes into the zeroth element of that array
void copyState(JNIEnv *jnienv, jobject intArray, int returncode) {
	int i;
	jint *body; // pointer to the fixed Java array //
	body = (* jnienv)->GetIntArrayElements(jnienv,intArray,0);

	body[0] = returncode;
	for (i=0; i<16; i++) {
		body[i+1] = Sensors.sonar[i];
	}
	body[17] = Sensors.xPos;
	body[18] = Sensors.yPos;
	body[19] = Sensors.angle;
	body[20] = Sensors.rVel;
	body[21] = Sensors.lVel;
	body[22] = Sensors.bumper;
	(* jnienv)->ReleaseIntArrayElements(jnienv,intArray,body,0);
}

JNIEXPORT jint JNICALL Java_RobotController_openSerial
(JNIEnv * jnienv, jobject javaobj) {

	int returnval;
	returnval=OpenSerial();
	return returnval;
}

JNIEXPORT jint JNICALL Java_RobotController_closeSerial
(JNIEnv * jnienv, jobject javaobj) {
	int returnval;
	returnval = CloseSerial();
	return returnval;
}

JNIEXPORT jint JNICALL Java_RobotController_sonarsOn
(JNIEnv * jnienv, jobject javaobj, jobject intArray) {
	int retval;
	retval = SonarsOn();
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_sonarsOff
(JNIEnv * jnienv, jobject javaobj, jobject intArray) {
	int retval;
	retval = SonarsOff();
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_zero
(JNIEnv * jnienv, jobject javaobj, jobject intArray){
	int retval;
	retval = Zero();
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_limp
(JNIEnv * jnienv, jobject javaobj, jobject intArray){
	int retval;
	retval = Limp();
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_confAc
  (JNIEnv *jnienv, jobject javaobj, jint ta, 
  jint unused1, jint unused2, jobject intArray){
	int retval;
	retval = ConfAc(ta, unused1, unused2);
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_confTimeout
(JNIEnv * jnienv, jobject javaobj, jint timeoutval, jobject intArray){
	int retval;
	retval = ConfTimeout(timeoutval);
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_getState
(JNIEnv * jnienv, jobject javaobj, jobject intArray){
	int retval;
	retval = GS();
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_rMove
  (JNIEnv * jnienv, jobject javaobj, 
  jint rightvel, jint leftvel, jobject intArray){
	int retval;
	retval = VM(rightvel, leftvel, 0);
	copyState(jnienv, intArray, retval);
	return retval;
}

JNIEXPORT jint JNICALL Java_RobotController_confSonars
  (JNIEnv * jnienv, jobject javaobj, jint rate, 
  jobject sonarArrayList, jobject intArray){
	int orderlist[16];
	int i, retval;
	jint *body; // pointer to the fixed Java array //
	body = (* jnienv)->GetIntArrayElements(jnienv,sonarArrayList,0);
	for (i=0; i < 16; i++) {
		orderlist[i] = body[i];
	}
	(* jnienv)->ReleaseIntArrayElements(jnienv,intArray,body,0);

	retval = ConfSonars(rate, orderlist);
	copyState(jnienv, intArray, retval);
	return retval;
}

